@@ -1,6 +1,6 @@
 =head1 NAME
 
-  PDL::Dbg -- functions to support debugging of PDL scripts
+PDL::Dbg - functions to support debugging of PDL scripts
 
 =head1 SYNOPSIS
 
@@ -18,6 +18,7 @@ you're doing rapid prototyping of new PDL code, let's say inside the
 perldl shell.
 
 =cut
+
 #' fool emacs
 
 package PDL::Dbg;
@@ -116,16 +117,18 @@ sub px {
   print $title;
   print '-'x(length($title)+3)."\n";
   my ($pdl,$npdls,$key,$val,$info) = ((),0,"","","");
-  while (($key,$val) = each(%stab)) {
-    $pdl = ${"$package$key"};
-    # print info for all objects derived from this class
-    if (isa($pdl,$classname)) {
-      $npdls++;
-      $info = $pdl->info($PDL::Dbg::Infostr);
-      printf "\$%-11s %s %s\n",$key,$info,(ref($pdl) eq $classname ? '' :
-					 ref($pdl));
-      # also print classname for derived classes
-    }
+  # while (($key,$val) = each(%stab)) {
+  foreach $key ( sort { lc($a) cmp lc($b) } keys(%stab) ) {
+     $val = $stab{$key};
+     $pdl = ${"$package$key"};
+     # print info for all objects derived from this class
+     if (UNIVERSAL::isa($pdl,$classname)) {
+        $npdls++;
+        $info = $pdl->info($PDL::Dbg::Infostr);
+        printf "\$%-11s %s %s\n",$key,$info,(ref($pdl) eq $classname ? '' :
+           ref($pdl));
+        # also print classname for derived classes
+     }
   }
   print "no $classname objects in package $package\n"
        unless $npdls;
@@ -153,7 +156,7 @@ should be sent to the PDL mailing list perldl@jachw.hawaii.edu.
 
 =head1 AUTHOR
 
-Copyright(C) 1997 Christian Soeller (csoelle@sghms.ac.uk).
+Copyright(C) 1997 Christian Soeller (c.soeller@auckland.ac.nz).
 All rights reserved. There is no warranty. You are allowed
 to redistribute this software / documentation under certain
 conditions. For details, see the file COPYING in the PDL
